package com.yihu.hos;

import com.yihu.hos.config.BeanConfig;
import com.yihu.hos.interceptor.WebMvcInterceptor;
import com.yihu.hos.web.framework.constant.ServiceFlowConstant;
import com.yihu.hos.web.framework.util.GridFSUtil;
import org.apache.activemq.command.ActiveMQQueue;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.annotation.Bean;
import org.springframework.data.mongodb.gridfs.GridFsOperations;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurerAdapter;
import org.zbus.mq.server.MqServer;
import org.zbus.mq.server.MqServerConfig;

import javax.jms.Queue;

@SpringBootApplication
public class ESBApplication extends WebMvcConfigurerAdapter  implements CommandLineRunner {
    private BeanConfig configuration;

    @Autowired
    private GridFsOperations operations;

    @Autowired
    public void setConfiguration(BeanConfig configuration) {
        this.configuration = configuration;
    }

    public static void main(String[] args) throws Exception {
        SpringApplication application = new SpringApplication(ESBApplication.class);
        application.run(args);
    }

    // 增加拦截器
    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        GridFSUtil.gridFsOperations = operations;
        registry.addInterceptor(new WebMvcInterceptor());
    }

    @Bean
    public Queue queue() {
        return new ActiveMQQueue(ServiceFlowConstant.FLOW_EVENT_QUEUE);
    }

    @Bean(name = "shellQueue")
    public Queue shellQueue() {
        return new ActiveMQQueue(ServiceFlowConstant.SHELL_EVENT_QUEUE);
    }

    @Bean(name = "mycatQueue")
    public Queue mycatQueue() {
        return new ActiveMQQueue(ServiceFlowConstant.MYCAT_EVENT_QUEUE);
    }

    @Override
    public void run(String... strings) throws Exception {
        MqServerConfig config = new MqServerConfig();
        config.serverPort = configuration.getZbusPort();
        config.storePath = configuration.getZbusStore();
        final MqServer server = new MqServer(config);
        server.start();
    }
}
